<?php

namespace Drupal\splio_utils\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;

/**
 * Class SplioTriggerResource.
 *
 * Provides a REST service for the SplioTriggerManager service.
 *
 * @RestResource(
 *   id = "splio_trigger_manager",
 *   label = @Translation("Splio trigger manager"),
 *   uri_paths = {
 *     "canonical" = "/splio/trigger",
 *     "create" = "/splio/trigger",
 *   }
 * )
 */
class SplioTriggerResource extends ResourceBase {

  /**
   * Sends a trigger to Splio so the platform sends a message to the recipients.
   *
   * @param array $data
   *   The trigger data to be fired in Splio.
   *   [
   *      message => 'xxxxx',
   *      opcode => [
   *        type => 'xxxx',
   *        success_message => 'xxx',
   *        failure_message => 'xxx',
   *      ],
   *   ].
   *
   * @return \Drupal\rest\ResourceResponse
   *   [
   *     code => HTTP response status code,
   *     error => has value if an error occurred
   *   ].
   */
  public function post(array $data): ResourceResponse {

    /** @var \Drupal\splio_utils\Services\SplioTriggerManager $triggerManager */
    $triggerManager = \Drupal::service('splio_utils.splio_trigger_manager');
    $response = $triggerManager->triggerMessage($data);
    $resourceResponse = new ResourceResponse();
    $resourceResponse->setStatusCode($response['code']);

    $resourceResponse->setContent(
      json_encode($response)
    );

    return $resourceResponse;
  }

}
