<?php

namespace Drupal\splio\Services;

use Drupal\splio\Entity\SplioEntity;

/**
 * Manages contacts lists data.
 */
class SplioContactsListsManager implements SplioContactsListsManagerInterface {

  /**
   * Splio entity handler service.
   *
   * @var \Drupal\splio\Services\SplioEntityHandlerInterface
   */
  private SplioEntityHandlerInterface $splioEntityHandler;

  public function __construct(SplioEntityHandlerInterface $splioEntityHandler) {

    $this->splioEntityHandler = $splioEntityHandler;
  }

  /**
   * Gets all contacts lists ids stored in the entity that sent to Splio.
   *
   * @param array $structure
   *   Data structure.
   *
   * @return array
   *   List of ids.
   */
  public function getIdListFromEntityStructure(array $structure): array {

    $structuredLists = $structure['lists'] ?? [];
    $idList = [];

    foreach ($structuredLists as $list) {
      $idList[] = (int) $list['id'];
    }

    return $idList;
  }

  /**
   * Gets all local contacts lists ids.
   *
   * @return array
   *   List of ids.
   */
  public function getIdsListFromStorage(): array {

    $idsList = [];
    $lists = $this->splioEntityHandler
      ->getLocalSplioEntitiesFields(['splio_entity' => SplioEntity::TYPE_CONTACTS_LISTS]);

    if (!empty($lists)) {
      /** @var \Drupal\splio\Entity\SplioFieldInterface $list */
      foreach ($lists as $list) {
        $idsList[] = (int) $list->getListIndex();
      }
    }

    return $idsList;
  }

}
